#!/opt/python/bin/python

# webwatch.py
#
# (c) November 2000, Greg Baker (greg.baker@ifost.org.au)
# 
# This program is released under the terms of the Gnu Public License
# I won't repeat the license here,  you know the deal -- the source
# code is always open.

# Usage:  
#   Typically you would run this from cron,  or schedule it through
#   VantagePoint,  or some such.
#
#   It takes any number of arguments -- they will be config files
#   describing what web sites are to be checked,  and with what
#   options.  
#   If no files are specified,  it will default to using
#   /etc/opt/webwatch/webwatch.conf   

#   It will check each site in turn,  and then exit.
#
# If you find this program useful,  I wouldn't mind you letting me know.
# If you want some help implementing it (hah hah),  I'm a mostly independant
# consultant,  and I'm always interested in additional work.
#
# Suggest additional features if you want to.
#
# ToDo:  
#   - Monitoring of HTTPS sites
#   - Monitoring where we check that the content doesn't match some
#     regular expression (e.g.  the site shouldn't say "error")
#   - Threading,  rather than forking
#   - Kicking the jobs off in parallel
#   - Using the pyopc (VantagePoint - Python integration) module instead
#     of starting opcmsg
#   - Be a little less HP-UX specfic 
#   - Better OpenView integration.  (e.g.  write itself as an ARF)
#   - Better NNM integration (i.e. it should be able to send an ovevent)
#   - (Future) FTP monitoring,  other TCP/IP service monitoring.
#   - (Future) Rewrite in C? C++? Eiffel?  OCaml?  Perl?

import ConfigParser
import sys
import time
import os

 #################################################################

defaults = {}
defaults['timeout'] = '7'  #seconds
defaults['mintimeout'] = '0.9'  #seconds -- how long to wait between requests
defaults['path'] = "/index.html"
defaults['port'] = '80'
defaults['application'] = 'Web server'
defaults['object'] = 'http://%(server)s:%(port)s%(path)s'
defaults['msg_text'] = 'Web site on %(server)s unreachable after %(timeout)s seconds'
defaults['msg_grp'] = 'Network'
defaults['node'] = '%(server)s'
defaults['action'] = "/opt/OV/bin/OpC/opcmsg severity=normal application='%(application)s' object='%(object)s' msg_text='%(msg_text)s' msg_grp='%(msg_grp)s' node='%(node)s'"

 ####################################################################

cfg = ConfigParser.ConfigParser(defaults)
if (len(sys.argv)>1):
  cfg.read(sys.argv[1:])
else:
  cfg.read("/etc/opt/webwatch/webwatch.conf")
 
 ######################################################################

from httplib import HTTP

for section in cfg.sections():
  server = cfg.get(section,"server")
  path = cfg.get(section,"path")
  port = cfg.getint(section,"port")
  pid = os.fork()
  if (pid == 0):
     try:
       h = HTTP(server,port)
       h.putrequest('GET',path)
       h.endheaders()
       errcode, errmsg, headers = h.getreply()
     except:
       # any problem here is bad.
       action = cfg.get(section,"action")
       os.system(action)
     sys.exit(0)
  time.sleep(cfg.getfloat(section,"mintimeout"))
  result=os.waitpid(pid,os.WNOHANG)
  if (result != (0,0)):
     continue
  time.sleep(cfg.getfloat(section,"timeout")-cfg.getfloat(section,"mintimeout"))
  os.waitpid(pid,os.WNOHANG)
  try:
    os.kill(pid,9)
    # if we haven't thrown an an exception,  that worked! ouch!
    action = cfg.get(section,"action")
    os.system(action)
  except OSError:
    # obviously completed within the time-frame
    pass

